@@ -1,8 +1,11 @@
 import abc
 import asyncio
 import functools
-from typing import Callable, TypeVar
+import warnings
+from typing import Callable, Dict, TypeVar, Union
 
+from sqlalchemy.ext.asyncio import AsyncEngine
+from sqlalchemy.future import Engine
 from sqlalchemy.orm import scoped_session
 
 try:
@@ -23,17 +26,19 @@ except ImportError:
 
 
 class AbcAsyncDatabase(metaclass=abc.ABCMeta):  # noqa: B024
-    def __new__(cls, engine, *args, **kwargs):
+
+    _instances: Dict[str, "AbcAsyncDatabase"] = None
+
+    def __new__(cls, engine: Union[Engine, AsyncEngine], *args, **kwargs):
         """Create a new instance of the database class.Each engine url corresponds to a database instance,
         and if it already exists, it is directly returned, otherwise a new instance is created.
         """
-        if not hasattr(cls, "_instances"):
-            cls._instances = {}
+        cls._instances = cls._instances or {}
         if engine.url not in cls._instances:
             cls._instances[engine.url] = super().__new__(cls)
         return cls._instances[engine.url]
 
-    def __init__(self) -> None:
+    def __init__(self, engine: Union[Engine, AsyncEngine], *args, **kwargs) -> None:
         for func_name in {
             "run_sync",
             "begin",
@@ -81,6 +86,35 @@ class AbcAsyncDatabase(metaclass=abc.ABCMeta):  # noqa: B024
             setattr(self, f"async_{func_name}", func)
 
     async def asgi_dispatch(self, request, call_next):
+        """
+        This method has been deprecated and is not recommended. Please use the `asgi_middleware` method instead.
+        Reference: https://www.starlette.io/middleware/#limitations
+        """
+        # 打印警告信息
+        warnings.warn(
+            "This method has been deprecated and is not recommended. Please use the `asgi_middleware` method instead.",
+            DeprecationWarning,
+        )
+        if request.scope.get(f"__sqlalchemy_database__:{id(self)}", False):
+            return await call_next(request)
+        # bind session to request
+        async with self.__call__(scope=id(request.scope)):
+            request.scope[f"__sqlalchemy_database__:{id(self)}"] = self
+            return await call_next(request)
+
+    def attach_middleware(self, app):
+        """Attach the middleware to the ASGI application.
+        Example:
+            ```Python
+            app = FastAPI()
+            db = Database.create("sqlite:///test.db")
+            db.attach_middlewares(app)
+            ```
+        """
+        app.add_middleware(self.asgi_middleware)
+
+    @property
+    def asgi_middleware(self):
         """Middleware for ASGI applications, such as: Starlette, FastAPI, Quart, Sanic, Hug, Responder, etc.
         Bind a SQLAlchemy session connection to the incoming HTTP request session context,
         you can access the session object through `self.session`.
@@ -89,12 +123,20 @@ class AbcAsyncDatabase(metaclass=abc.ABCMeta):  # noqa: B024
             ```Python
             app = FastAPI()
             db = Database.create("sqlite:///test.db")
-            app.add_middleware(BaseHTTPMiddleware, dispatch=db.asgi_dispatch)
+            app.add_middleware(db.asgi_middleware)
             ```
         """
-        if request.scope.get(f"__sqlalchemy_database__:{id(self)}", False):
-            return await call_next(request)
-        # bind session to request
-        async with self.__call__(scope=id(request.scope)):
-            request.scope[f"__sqlalchemy_database__:{id(self)}"] = self
-            return await call_next(request)
+
+        def asgi_decorator(app):
+            @functools.wraps(app)
+            async def wrapped_app(scope, receive, send):
+                if scope.get(f"__sqlalchemy_database__:{id(self)}", False):
+                    return await app(scope, receive, send)
+                    # bind session to request
+                async with self.__call__(scope=id(scope)):
+                    scope[f"__sqlalchemy_database__:{id(self)}"] = self
+                    await app(scope, receive, send)
+
+            return wrapped_app
+
+        return asgi_decorator
